"""This module contains the urls of the ``cms`` app"""

from macrosurl import url

from . import views


app_name = 'cms'

urlpatterns = [
    # Cms index
    url(r'^$', views.Index.as_view(), name='index'),
    # Discover item detail
    url(
        r'discover/:slug/',
        views.DiscoverItemDetailView.as_view(),
        name='discover_detail'
    ),
]



"""This module holds the views related to the ``QuestionWall`` model"""

from django.utils.translation import ugettext_lazy as _
from django.views import generic

from mtp.apps.common.views_utils import AjaxResponseMixin
from mtp.apps.trainings.views.activities import (
    TrainingActivityCreateBase,
    TrainingActivityDetailBase,
    TrainingActivityEditBase,
)

from ..forms import QuestionWallForm
from ..models import QuestionWall


class QuestionWallBase(TrainingActivityDetailBase):
    """Base view to use on all views related to the ``QuestionWall`` model.

    Define the queryset to use for the current user.

    """

    def get_queryset(self):
        """Get all ``QuestionWall`` instances the current user is allowed to see.

        The user can only see questionwall instance he own.
        Also excludes deleted questionwall from the result.

        Returns
        -------
        Queryset of ``mtp.apps.ideas.models.QuestionWall``
            A lazy queryset with the wanted questionwalls.

        """

        return super().get_queryset().instance_of(QuestionWall)

    def get_success_url(self):
        """Get the url where to go back when an action is done on a ``QuestionWall``"""

        return self.object.get_detail_url()


class QuestionWallEditBase(AjaxResponseMixin, QuestionWallBase):
    """Base view to create or edit a given instance of ``QuestionWall``"""

    model = QuestionWall
    form_class = QuestionWallForm
    pk_url_kwarg = 'questionwall_uuid'


class QuestionWallDetailView(QuestionWallEditBase, generic.DetailView):
    """Detail View for a ``Brainstorming`` instance"""

    template_name = 'ideas/idea_activity_detail.html'


class QuestionWallCreateView(QuestionWallEditBase, TrainingActivityCreateBase):
    """View to create a new ``QuestionWall`` instance"""

    success_message = _('Your Questionwall was successfully created.')


class QuestionWallUpdateView(QuestionWallEditBase, TrainingActivityEditBase):
    """View to update an existing ``QuestionWall`` instance"""

    success_message = _('Your Questionwall was successfully updated.')



"""This module contains the management methods for the ``common`` app"""

import operator

from django.apps import apps
from django.contrib.contenttypes.management import get_contenttypes_and_models
from django.db import DEFAULT_DB_ALIAS


def create_ordered_contenttypes(app_config):
    """Create the contenttypes for the given ``app_config``

    Parameters
    ----------
    app_config: django ``AppConfig``
        The config of the app for which we want to create the contenttypes

    For the other parameters, see the ``create_contenttypes`` django method.

    Notes
    -----
    The django method ``create_contenttypes`` is connected to the ``post_migrate`` signal.
    In our given context MASTER/SLAVE, the way of django doing have two flaws:

    1 - The ``post_migrate`` signal is called after the ``migrate`` command and it might lead
        to creation of the contenttypes "on the fly" during a data migration.
    2 - The django method don't order the models when inserting the contenttypes.

    This two flaws lead to one big one: The contenttypes between the SLAVES and the MASTER
    might vary, we might get a different contenttypes id for the same model.

    So we create our own custom method and call it manually when needed.

    See Also
    --------
    ``django.contrib.contenttypes.management.create_contenttypes``

    """

    ContentType = apps.get_model('contenttypes', 'ContentType')  # pylint: disable=invalid-name
    content_types, app_models = get_contenttypes_and_models(
        app_config,
        DEFAULT_DB_ALIAS,
        ContentType
    )

    # Order the app models by their names
    app_models = sorted(app_models.items(), key=operator.itemgetter(0))

    cts = [
        ContentType(
            app_label=app_config.label,
            model=model_name,
        )
        for (model_name, model) in app_models
        if model_name not in content_types
    ]
    ContentType.objects.using(DEFAULT_DB_ALIAS).bulk_create(cts)



"""This module holds the views related to the ``Brainstorming`` model"""

from django.utils.translation import ugettext_lazy as _
from django.views import generic

from mtp.apps.common.views_utils import AjaxResponseMixin
from mtp.apps.trainings.views.activities import (
    TrainingActivityCreateBase,
    TrainingActivityDetailBase,
    TrainingActivityEditBase,
)

from ..forms import BrainstormingForm
from ..models import Brainstorming


class BrainstormingBase(TrainingActivityDetailBase):
    """Base view to use on all views related to the ``Brainstorming`` model.

    Define the queryset to use for the current user.

    """

    def get_queryset(self):
        """Get all ``Brainstorming`` instances the current user is allowed to see.

        The user can only see brainstorming instance he own.
        Also excludes deleted brainstorming from the result.

        Returns
        -------
        Queryset of ``mtp.apps.ideas.models.Brainstorming``
            A lazy queryset with the wanted brainstorming.

        """

        return super().get_queryset().instance_of(Brainstorming)

    def get_success_url(self):
        """Get the url where to go back when an action is done on a ``Brainstorming``"""

        return self.object.get_detail_url()


class BrainstormingEditBase(AjaxResponseMixin, BrainstormingBase):
    """Base view to create or edit a given instance of ``Brainstorming``"""

    model = Brainstorming
    form_class = BrainstormingForm
    pk_url_kwarg = 'brainstorming_uuid'


class BrainstormingDetailView(BrainstormingEditBase, generic.DetailView):
    """Detail View for a ``Brainstorming`` instance"""

    template_name = 'ideas/idea_activity_detail.html'


class BrainstormingCreateView(BrainstormingEditBase, TrainingActivityCreateBase):
    """View to create a new ``Brainstorming`` instance"""

    success_message = _('Your brainstorming was successfully created.')


class BrainstormingUpdateView(BrainstormingEditBase, TrainingActivityEditBase):
    """View to update an existing ``Brainstorming`` instance"""

    success_message = _('Your brainstorming was successfully updated.')



"""This module contains the views of the ``cms`` app"""

from random import shuffle
from urllib.parse import urlparse

from django.core.exceptions import ObjectDoesNotExist
from django.conf import settings
from django.db import models
from django.urls import resolve, Resolver404, reverse
from django.utils.translation import get_language
from django.views.generic import DetailView, TemplateView

from django_discover.models import DiscoverItem

from mtp.apps.common.views_utils import LoginRequiredMixin
from mtp.apps.trainings.models import Training, TrainingActivity


class DiscoverItemsQuerySetMixin:
    """View mixin to get the ``DiscoverItem`` for the current version"""

    @staticmethod
    def get_discover_item_queryset():
        """Get the ``DiscoverItem`` for the current language"""

        # Filter for the current version
        discover_item_queryset_for_version = DiscoverItem.objects.filter(
            is_active=True
        )

        # Filter for the current language
        discover_item_queryset = discover_item_queryset_for_version.for_language(
            get_language()
        )

        # If no discover item exists for the current language, fallback to the default language
        if not discover_item_queryset.exists():
            discover_item_queryset = discover_item_queryset_for_version.for_language(
                settings.DEFAULT_NATURAL_LANGUAGE
            )

        return discover_item_queryset


class Index(LoginRequiredMixin, TemplateView, DiscoverItemsQuerySetMixin):
    """Index page for the cms"""

    template_name = 'cms/index.html'

    def get_randomized_discover_items(self):
        """Get the randomized discover items"""

        discover_items_as_list = list(self.get_discover_item_queryset().annotate(
            sort_order=models.Case(
                models.When(tag=DiscoverItem.TAG_CHOICES.IMPORTANT, then=0),
                models.When(tag=DiscoverItem.TAG_CHOICES.NEW, then=1),
                default=2,
                output_field=models.IntegerField()
            )
        ))

        # Firstly, randomize the list
        shuffle(discover_items_as_list)

        # Secondly, order the item per the "sort_order" key
        return sorted(discover_items_as_list, key=lambda x: x.sort_order)

    def get_context_data(self, **kwargs):
        """Add the ``current_main_section`` to the context

        Parameters
        ----------
        For the parameters, see ``django.forms.generic.base.ContextMixin``

        """

        return super().get_context_data(
            current_main_section='index',
            discover_items=self.get_randomized_discover_items(),
            **kwargs
        )


class DiscoverItemDetailView(LoginRequiredMixin, DetailView, DiscoverItemsQuerySetMixin):
    """Detail view to display a single ``DiscoverItem``"""

    template_name = 'cms/discover/discover_item_detail.html'

    def get_queryset(self):
        """Filter the ``DiscoverItem`` queryset"""

        return self.get_discover_item_queryset()

    def get_contextual_discover_item_url(self):
        """Get the current discover item link url if applicable

        Some discover item don't have their "link" attributes filled, so we should
        build manually the link if applicable (ie: for the training activity-related items)

        Returns
        -------
        str or None
            The link_url for the discover item,
            or None.

        """

        activity_types = {
            activity_model.__name__.lower(): activity_model
            for activity_model in TrainingActivity.available_activity_models()
        }
        urls_to_match = {
            'cms:index': 'trainings:activities-private-library',
            'trainings:training-detail': 'trainings:training-detail',
            'trainings:activities-private-library': 'trainings:activities-private-library'
        }

        # The current ``DiscoverItem`` is not related to one of the ``TrainingActivity``,
        # return the default context
        if self.object.slug not in activity_types.keys():
            return

        try:
            # Try to match the referer to an url
            match = resolve(urlparse(self.request.META.get('HTTP_REFERER', None))[2])
            resolved_url = '%s:%s' % (match.namespace, match.url_name)

            if 'training_uuid' in match.kwargs:
                training = Training.objects.non_deleted().get(
                    id=match.kwargs['training_uuid']
                )
                create_url = activity_types[self.object.slug].get_training_create_url(
                    training=training
                )
            else:
                create_url = activity_types[self.object.slug].get_create_url()

            # Finally build the full url (we want to redirect the user AND open a modal)
            return '%s?modal-url=%s' % (
                reverse(urls_to_match[resolved_url], kwargs=match.kwargs),
                create_url
            )

        except (Resolver404, KeyError, ObjectDoesNotExist):
            return

    def get_context_data(self, **kwargs):
        """Add the discover item action url link and title to the context if applicable

        Parameters
        ----------
        For the parameters, see ``django.forms.generic.base.ContextMixin``

        """

        item_link = self.get_contextual_discover_item_url()

        if not item_link:
            return super().get_context_data(**kwargs)

        return super().get_context_data(
            discover_link_url=item_link,
            **kwargs
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0004_djangotask_state'),
    ]

    operations = [
        migrations.AlterField(
            model_name='djangotask',
            name='user',
            field=models.ForeignKey(related_name='tasks', verbose_name='User', to=settings.AUTH_USER_MODEL, null=True),
        ),
    ]



"""This module contains the forms for the ``ideas`` app"""

from django import forms
from django.utils.translation import ugettext_lazy as _

from mtp.apps.trainings.forms import TrainingActivityForm

from .models import Brainstorming, QuestionWall, WordCloud


class BaseIdeaActivityForm(TrainingActivityForm):
    """Base Form for the ``ideas`` activity models"""

    is_anonymous = forms.BooleanField(
        required=False,
        label=_('Anonymous')
    )

    class Meta:
        fields = TrainingActivityForm.Meta.fields + (
            'is_anonymous',
        )


class BrainstormingForm(BaseIdeaActivityForm):
    """Form for the ``Brainstorming`` model"""

    class Meta:
        fields = BaseIdeaActivityForm.Meta.fields
        model = Brainstorming


class QuestionWallForm(BaseIdeaActivityForm):
    """Form for the ``QuestionWall`` model"""

    class Meta:
        fields = BaseIdeaActivityForm.Meta.fields
        model = QuestionWall


class WordCloudForm(BaseIdeaActivityForm):
    """Form for the ``WordCloud`` model"""

    class Meta:
        fields = BaseIdeaActivityForm.Meta.fields
        model = WordCloud




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('assessments', '0021_userlessonrecommendation_score'),
    ]

    operations = [
        migrations.AddField(
            model_name='recommendationlevel',
            name='is_reference',
            field=models.BooleanField(default=False),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('assessments', '0020_certificationstatus_is_reference'),
    ]

    operations = [
        migrations.AddField(
            model_name='userlessonrecommendation',
            name='score',
            field=models.PositiveIntegerField(default=0),
            preserve_default=False,
        ),
    ]



"""This module holds the tests for the views of the ``cms`` app"""

from django.conf import settings
from django.core.urlresolvers import reverse

from django_discover.factories import DiscoverItemFactory
from django_discover.models import DiscoverItem

from mtp.apps.common.tests.test_cases import MTPTestCase
from mtp.apps.questionnaires.models import Quiz


class CMSViewTestCase(MTPTestCase):
    """Test cases for the views of the ``cms`` app"""

    def setUp(self):
        """Login as an admin"""

        super().setUp()

        self.login_as_admin()

    def test_cms_index_view_should_work(self):
        """Test that the cms ``Index`` view is working correctly"""

        response = self.client.get(reverse('cms:index'))
        self.assertEqual(response.status_code, 200)

    def test_cms_index_view_should_contains_ordered_discover_items(self):
        """Test that the index should contains the ordered discover items"""

        # Create 3 discover items
        important_item = DiscoverItemFactory(
            language=settings.DEFAULT_NATURAL_LANGUAGE,
            tag=DiscoverItem.TAG_CHOICES.IMPORTANT
        )
        new_item = DiscoverItemFactory(
            language=settings.DEFAULT_NATURAL_LANGUAGE,
            tag=DiscoverItem.TAG_CHOICES.NEW
        )
        item = DiscoverItemFactory(language=settings.DEFAULT_NATURAL_LANGUAGE)

        response = self.client.get(reverse('cms:index'))
        self.assertEqual(
            list(response.context['discover_items']),
            [
                important_item,
                new_item,
                item
            ]
        )


class DiscoverItemDetailView(MTPTestCase):
    """Test case for the ``DiscoverItemDetailView`` view"""

    @classmethod
    def setUpTestData(cls):
        """Create some ``DiscoverItems`` to test against"""

        super().setUpTestData()

        cls.default_discover_item = DiscoverItemFactory(
            slug='default_discover_item',
            language=settings.DEFAULT_NATURAL_LANGUAGE
        )

        cls.activity_discover_item = DiscoverItemFactory(
            slug='quiz',
            language=settings.DEFAULT_NATURAL_LANGUAGE
        )

    def setUp(self):
        """Login as an admin"""

        super().setUp()

        self.login_as_admin()

    def test_view_should_output_404_if_no_matching_discover_item_were_found(self):
        """Test that the view should output a 404 if no matching items were found"""

        response = self.client.get(
            reverse('cms:discover_detail', kwargs={'slug': 'not-a-slug'})
        )

        self.assertEqual(response.status_code, 404)

    def test_view_should_work_if_default_discover_item(self):
        """Test that the view should work if passing a slug from a discover item"""

        response = self.client.get(
            reverse('cms:discover_detail', kwargs={'slug': self.default_discover_item.slug})
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.context['discoveritem'],
            self.default_discover_item
        )

    def test_view_should_fallback_to_default_language_discover_item(self):
        """Test that the view should fallback to the default language"""

        # Set the language to "fr"
        self.client.cookies.load({settings.LANGUAGE_COOKIE_NAME: 'fr'})

        response = self.client.get(
            reverse('cms:discover_detail', kwargs={'slug': self.default_discover_item.slug})
        )

        self.assertEqual(response.status_code, 200)

        # Because there is no discover item available for french, we get the english one
        self.assertEqual(
            response.context['discoveritem'],
            self.default_discover_item
        )

        # Create the french one
        french_discover_item = DiscoverItemFactory(
            slug='default_discover_item',
            language='fr'
        )

        response = self.client.get(
            reverse('cms:discover_detail', kwargs={'slug': self.default_discover_item.slug})
        )

        # Now that it exists, we should get the french one
        self.assertEqual(
            response.context['discoveritem'],
            french_discover_item
        )

    def test_view_should_add_the_contextual_discover_item_url_to_the_context(self):
        """Test that the view should add the discover item url to the context if applicable"""

        # Testing from the index page...
        response = self.client.get(
            reverse(
                'cms:discover_detail',
                kwargs={'slug': self.activity_discover_item.slug},
            ),
            HTTP_REFERER=reverse('cms:index')
        )

        self.assertEqual(response.status_code, 200)

        # ...We should have a link to the private library (and the "modal-url" param)
        self.assertEqual(
            response.context['discover_link_url'],
            '%s?modal-url=%s' % (
                reverse('trainings:activities-private-library'),
                Quiz.get_create_url(),
            )
        )

        # Testing from the private library page...
        response = self.client.get(
            reverse(
                'cms:discover_detail',
                kwargs={'slug': self.activity_discover_item.slug},
            ),
            HTTP_REFERER=reverse('trainings:activities-private-library')
        )

        self.assertEqual(response.status_code, 200)

        # ...We should have a link to the private library (and the "modal-url" param)
        self.assertEqual(
            response.context['discover_link_url'],
            '%s?modal-url=%s' % (
                reverse('trainings:activities-private-library'),
                Quiz.get_create_url(),
            )
        )

        # Testing from the training detail url page...
        training_url = reverse('trainings:training-detail', kwargs={
            'training_uuid': self.admin_training.id
        })

        response = self.client.get(
            reverse(
                'cms:discover_detail',
                kwargs={'slug': self.activity_discover_item.slug},
            ),
            HTTP_REFERER=training_url
        )

        self.assertEqual(response.status_code, 200)

        # ...We should have a link to the private library (and the "modal-url" param)
        self.assertEqual(
            response.context['discover_link_url'],
            '%s?modal-url=%s' % (
                training_url,
                Quiz.get_training_create_url(self.admin_training),
            )
        )



"""This module contains the forms for the ``photostories`` app"""

from mtp.apps.trainings.forms import TrainingActivityForm

from .models import Photostory


class PhotostoryForm(TrainingActivityForm):
    """Form for the ``Photostory`` model"""

    class Meta:
        fields = TrainingActivityForm.Meta.fields
        model = Photostory



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('translation', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='translatablelanguage',
            name='first_update',
            field=models.ForeignKey(related_name='translation_translatablelanguage_by_first', null=True, to='sync.Update', blank=True),
        ),
        migrations.AlterField(
            model_name='translatablelanguage',
            name='last_update',
            field=models.ForeignKey(related_name='translation_translatablelanguage_by_last', null=True, to='sync.Update', blank=True),
        ),
        migrations.AlterField(
            model_name='translation',
            name='first_update',
            field=models.ForeignKey(related_name='translation_translation_by_first', null=True, to='sync.Update', blank=True),
        ),
        migrations.AlterField(
            model_name='translation',
            name='last_update',
            field=models.ForeignKey(related_name='translation_translation_by_last', null=True, to='sync.Update', blank=True),
        ),
    ]
